"""
Unit of member instance.
"""
from urllib.parse import urljoin


class Unit:
    """
    Unit object including member,team and role.
    """

    def __init__(self, space: 'Space'):
        self._space = space

    def _describe_member_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/members/{unit_id}")

    @property
    def _role_endpoint(self):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/roles")

    def _describe_role_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/roles/{unit_id}")

    def _list_role_unit_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/roles/{unit_id}/units")

    @property
    def _team_endpoint(self):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/teams")

    def _describe_team_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/teams/{unit_id}")

    def _list_sub_team_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/teams/{unit_id}/children")

    def _list_team_member_endpoint(self, unit_id: str):
        return urljoin(self._space.apitable.api_base,
                       f"/fusion/v1/spaces/{self._space.id}/teams/{unit_id}/members")



import time
import unittest
import warnings
from vika import Vika
from vika.types import EmbedLinkThemeEnum,EmbedLinkPermissionType
from . import DOMAIN, TOKEN, SPACE_ID, DATASHEET_ID


class TestEmbedLinkCRD(unittest.TestCase):
    def setUp(self):
        warnings.simplefilter("ignore", ResourceWarning)
        apitable = Vika(TOKEN)
        apitable.set_api_base(DOMAIN)
        self.spc = apitable.space(SPACE_ID)

    def test_embed_link_crd(self):
        datasheet = self.spc.datasheet(DATASHEET_ID)
        newEmbedLink = datasheet.create_embed_link(
            {
                "paylod": {
                    "primarySideBar": {"collapsed": False},
                    "viewControl": {
                        "viewId": "viwBBc0PMN3WB",
                        "tabBar": False,
                        "toolBar": {
                            "basicTools": False,
                            "shareBtn": False,
                            "widgetBtn": False,
                            "apiBtn": False,
                            "formBtn": False,
                            "historyBtn": False,
                            "robotBtn": False,
                        },
                        "collapsed": False,
                    },
                    "bannerLogo": True,
                    "permissionType": EmbedLinkPermissionType.READ_ONLY,
                },
                "theme": EmbedLinkThemeEnum.Light,
            }
        )
        self.assertIsNotNone(newEmbedLink.linkId)
        time.sleep(0.5)
        embedLinks = datasheet.get_embed_links()
        self.assertEqual(len(embedLinks), 1)
        time.sleep(0.5)
        isDeleted = datasheet.delete_embed_link(newEmbedLink.linkId)
        self.assertTrue(isDeleted)
        time.sleep(0.5)
        embedLinks = datasheet.get_embed_links()
        self.assertEqual(len(embedLinks), 0)

    def tearDown(self):
        """Create an embed link test class object destroy hook:"""


if __name__ == "__main__":
    unittest.main()



"""
Unit of role instance.
"""
from vika.types.unit_model import ModifyRoleRequest, RawRole, PaginationRole, CreateRoleRequest, RoleUnit
from vika.unit import Unit


class Role(Unit):

    def create(self, data: CreateRoleRequest) -> RawRole:
        """
        Add a role to the specified space.
        @param data: one or multipart property of name,sequence
        @return: RawRole
        """
        r = self._space.apitable.request.post(self._role_endpoint,
                                              json=dict(data)).json()
        if r["success"]:
            return RawRole(**r['data']['role'])
        else:
            raise Exception(r['message'])

    def update(self, unit_id: str, data: ModifyRoleRequest) -> RawRole:
        """
        Update role's name, sequence and roles information.
        @param data: one or multipart property of name,sequence
        @param unit_id: role's unit id
        @return: RawRole
        """
        r = self._space.apitable.request.put(self._describe_role_endpoint(unit_id),
                                             json=dict(data)).json()
        if r["success"]:
            return RawRole(**r['data']['role'])
        else:
            raise Exception(r["message"])

    def delete(self, unit_id: str) -> bool:
        """
        Remove a role from the specified space, notice: Only empty roles can be deleted，
        @param unit_id: role's unit id
        @return: bool
        """
        r = self._space.apitable.request.delete(self._describe_role_endpoint(unit_id)).json()
        return r['success']

    def list(self, page_num=1, page_size=100) -> PaginationRole:
        """
        Paging to get the list of roles.
        @param page_num: the page number of the pagination
        @param page_size: this parameter only accepts integers from 1-1000
        """
        r = self._space.apitable.request.get(self._role_endpoint,
                                             params={
                                                 'pageNum': page_num,
                                                 'pageSize': page_size
                                             }).json()
        if r["success"]:
            return PaginationRole(**r['data'])
        else:
            raise Exception(r["message"])

    def list_unit(self, unit_id: str) -> RoleUnit:
        """
        Get the organizational units under the specified role unitId, the returned data includes teams and members.
        @param unit_id: role's unit id
        """
        r = self._space.apitable.request.get(self._list_role_unit_endpoint(unit_id)).json()
        if r["success"]:
            return RoleUnit(**r['data'])
        else:
            raise Exception(r["message"])



"""
Unit of member instance.
"""

from vika.types.unit_model import ModifyMemberRequest, RawMember
from vika.unit.unit import Unit


class Member(Unit):
    """
    Member object.
    """

    def update(self, unit_id: str, data: ModifyMemberRequest) -> RawMember:
        """
        Update member's name, teams and roles information.
        @param data: one or multipart property of name,teams,roles.
        @param unit_id: role's unit id
        @return: RawMember
        """
        r = self._space.apitable.request.put(self._describe_member_endpoint(unit_id),
                                             json=dict(data)).json()
        if r["success"]:
            return RawMember(**r['data']['member'])
        else:
            raise Exception(r["message"])

    def get(self, unit_id: str) -> RawMember:
        """
        Update member's name, teams and roles information.
        @param unit_id: role's unit id
        @return: RawMember
        """
        r = self._space.apitable.request.get(self._describe_member_endpoint(unit_id)).json()
        if r["success"]:
            return RawMember(**r['data']['member'])
        else:
            raise Exception(r["message"])

    def delete(self, unit_id: str) -> bool:
        """
        Remove a member from the specified space
        @param unit_id: role's unit id
        @return: bool
        """
        r = self._space.apitable.request.delete(self._describe_member_endpoint(unit_id)).json()
        return r['success']
